package cn.fw.dalaran.common.utils;

import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONUtil;
import com.google.common.base.Joiner;
import com.google.common.base.Splitter;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import org.bytedeco.javacpp.Loader;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.io.*;
import java.net.URL;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

@Slf4j
public class ImageUtils {

    /**
     * 修改图片分辨率
     *
     * @param imagePath 原图片地址
     * @param outputDir 输出目录
     * @param width     目标宽度
     * @param height    目标高度
     * @return 图片文件路径
     * @throws Exception 异常
     */
    public static String modifyResolution(String imagePath, String outputDir, Integer width, Integer height) throws Exception {
        List<String> paths = Splitter.on(".").splitToList(imagePath);
        String ext = paths.get(paths.size() - 1);
        if (!Arrays.asList("jpg", "png").contains(ext)) {
            throw new Exception("format error");
        }
        String resultPath =
                Joiner.on(File.separator).join(Arrays.asList(outputDir, IdUtil.simpleUUID() + "." + ext));
        String ffmpeg = Loader.load(org.bytedeco.ffmpeg.ffmpeg.class);
        ProcessBuilder builder =
                new ProcessBuilder(
                        ffmpeg,
                        "-i",
                        imagePath,
                        "-vf",
                        MessageFormat.format("scale={0}:{1}", String.valueOf(width), String.valueOf(height)),
                        resultPath);
        builder.inheritIO().start().waitFor();
        return resultPath;
    }

    /**
     * 修改图片分辨率(重载)
     *
     * @param imagePath 原图片地址
     * @param outputDir 输出目录
     * @param fileName  指定文件名前缀
     * @param width     目标宽度
     * @param height    目标高度
     * @return 图片文件路径
     * @throws Exception 异常
     */
    public static String modifyResolution(String imagePath, String outputDir, String fileName, Integer width, Integer height) throws Exception {
        List<String> paths = Splitter.on(".").splitToList(imagePath);
        String ext = paths.get(paths.size() - 1);
        if (!Arrays.asList("jpg", "png").contains(ext)) {
            throw new Exception("format error");
        }
        String resultPath =
                Joiner.on(File.separator).join(Arrays.asList(outputDir, fileName + "." + ext));
        File file = new File(outputDir);
        if (!file.exists()) {
            boolean result = file.mkdirs();
        }
        String ffmpeg = Loader.load(org.bytedeco.ffmpeg.ffmpeg.class);
        ProcessBuilder builder =
                new ProcessBuilder(
                        ffmpeg,
                        "-i",
                        imagePath,
                        "-vf",
                        MessageFormat.format("scale={0}:{1}", String.valueOf(width), String.valueOf(height)),
                        resultPath);
        builder.inheritIO().start().waitFor();
        return resultPath;
    }

    /**
     * 修改图片分辨率(重载)
     *
     * @param imagePath 原图片地址
     * @param outputDir 输出目录
     * @param fileName  指定文件名前缀
     * @param width     目标宽度
     * @param height    目标高度
     * @return 图片文件路径
     * @throws Exception 异常
     */
    public static String modifyResolution1(String imagePath, String outputDir, String fileName, Integer width, Integer height) throws Exception {
        log.info(String.format("ImageUtils.modifyResolution1()方法入参:\n imagePath: %s\n outputDir: %s\n fileName: %s\n width: %s, height: %s",
                imagePath, outputDir, fileName, width, height));
        List<String> paths = Splitter.on(".").splitToList(imagePath);
        String ext = paths.get(paths.size() - 1);
        if (!Arrays.asList("jpg", "png").contains(ext)) {
            throw new Exception("format error");
        }
        File file = new File(outputDir);
        if (!file.exists()) {
            boolean setWritable = file.setWritable(true);
            log.info(String.format("创建文件夹, 路径为: %s, 修改写权限%s, 创建结果为: %s", outputDir, setWritable ? "成功" : "失败", file.mkdirs() ? "成功" : "失败"));
        }
        String resultPath = outputDir + File.separator + fileName + "." + ext;
        Thumbnails.of(imagePath).size(width, height).toFile(resultPath);
        log.info(String.format("ImageUtils.modifyResolution1()返回结果为: %s", resultPath));
        return resultPath;
    }

    /**
     * 读取图片文件, 将图片像素转换成二维数组
     *
     * @param file 文件对象
     * @return 图片文件
     */
    public static int[][] readImagePixel(File file) {
        BufferedImage readImg = null;
        try {
            readImg = ImageIO.read(file);
        } catch (IOException e) {
            e.printStackTrace();
        }
        assert readImg != null;
        int width = readImg.getWidth();
        int height = readImg.getHeight();
        int[][] imgPixelArray = new int[width][height];
        for (int i = 0; i < width; i++) {
            for (int j = 0; j < height; j++) {
                imgPixelArray[i][j] = readImg.getRGB(i, j);
            }
        }
        return imgPixelArray;
    }

    /**
     * 将url中的图片转为File对象存放到当前环境temp中
     *
     * @param url url路径
     * @return File对象
     */
    public static File convertFileByUrl(String url) {
        File file = null;
        URL urlFile;
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try {
            file = File.createTempFile("fw_theme_cover_", ".png");
            //下载
            urlFile = new URL(url);
            inputStream = urlFile.openStream();
            outputStream = new FileOutputStream(file);
            int bytesRead;
            byte[] buffer = new byte[8192];
            while ((bytesRead = inputStream.read(buffer, 0, 8192)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (null != outputStream) {
                    outputStream.close();
                }
                if (null != inputStream) {
                    inputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return file;
    }

    /**
     * 将url中的图片转为File对象存放到当前环境temp中(重载)
     *
     * @param url url路径
     * @return File对象
     */
    public static File convertFileByUrl(String url, String itemId) {
        log.info(String.format("ImageUtils.convertFileByUrl()方法入参:\n url: %s\n itemId: %s", url, itemId));
        File file = null;
        URL urlFile;
        InputStream inputStream = null;
        OutputStream outputStream = null;
        try {
            file = File.createTempFile("fw_theme_cover_" + itemId, ".png");
            //下载
            urlFile = new URL(url);
            inputStream = urlFile.openStream();
            outputStream = new FileOutputStream(file);
            int bytesRead;
            byte[] buffer = new byte[8192];
            while ((bytesRead = inputStream.read(buffer, 0, 8192)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (null != outputStream) {
                    outputStream.close();
                }
                if (null != inputStream) {
                    inputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        log.info(String.format("ImageUtils.convertFileByUrl()返回结果为: %s", JSONUtil.toJsonStr(file)));
        return file;
    }

    /**
     * 获取RGB灰度
     *
     * @param numPixels 像素点
     * @return RGB灰度
     */
    public static int getRgbGray(int numPixels) {
        // byte -128 127
        int red = (numPixels >> 16) & 0xFF;
        int green = (numPixels >> 8) & 0xFF;
        int blue = numPixels & 0xFF;
        //int blue = (numPixels >> 0) & 0xFF;
        // 灰度 -- 减少计算量 以及 更方便计算
        return (red + green + blue) / 3;
    }

    /**
     * 根据二维数组 绘制图片
     *
     * @param graphics graphics对象
     * @param imgArray 图片像素数组
     */
    public static void drawImage(Graphics graphics, int[][] imgArray, int x, int y, ImageObserver observer) {
        BufferedImage bufferImg00 = new BufferedImage(imgArray.length, imgArray[0].length, BufferedImage.TYPE_INT_ARGB);
        for (int i = 0; i < imgArray.length; i++) {
            for (int j = 0; j < imgArray[i].length; j++) {
                bufferImg00.setRGB(i, j, imgArray[i][j]);
            }
        }
        // 一次性绘制出来 只占一次IO
        graphics.drawImage(bufferImg00, x, y, observer);
    }

    /**
     * 计算指纹
     *
     * @param imgArray
     */
    public static int[] getFingerprint(int[][] imgArray) {
        int[] rgbAverage = new int[64];
        int width = 8;
        int height = 8;
        int a1 = imgArray.length / width;
        int b1 = imgArray[0].length / height;
        int a2 = imgArray.length - a1 * (width - 1);
        int b2 = imgArray[0].length - b1 * (height - 1);
        int[][] average = new int[width][height];
        int sum0 = 0;
        for (int m = 0; m < width; m++) {
            for (int n = 0; n < height; n++) {
                int sum = 0;
                if (m != (width - 1) && n != (height - 1)) {
                    for (int i = m * a1; i < a1 + m * a1; i++) {
                        for (int j = n * b1; j < b1 + n * b1; j++) {
                            int num1 = imgArray[i][j];
                            int gray1 = getRgbGray(num1);
                            sum += gray1;
                        }
                    }
                    average[m][n] = sum / (a1 * b1);
                }
                if (m == (width - 1) && n != (height - 1)) {
                    for (int i = m * a1; i < a2 + m * a1; i++) {
                        for (int j = n * b1; j < b1 + n * b1; j++) {
                            int num1 = imgArray[i][j];
                            int gray1 = getRgbGray(num1);
                            sum += gray1;
                        }
                    }
                    average[m][n] = sum / (a2 * b1);
                }
                if (m != (width - 1) && n == (height - 1)) {
                    for (int i = m * a1; i < a1 + m * a1; i++) {
                        for (int j = n * b1; j < b2 + n * b1; j++) {
                            int num1 = imgArray[i][j];
                            int gray1 = getRgbGray(num1);
                            sum += gray1;
                        }
                    }
                    average[m][n] = sum / (a1 * b2);
                }
                if (m == (width - 1) && n == (height - 1)) {
                    for (int i = m * a1; i < a2 + m * a1; i++) {
                        for (int j = n * b1; j < b2 + n * b1; j++) {
                            int num1 = imgArray[i][j];
                            int gray1 = getRgbGray(num1);
                            sum += gray1;
                        }
                    }
                    average[m][n] = sum / (a2 * b2);
                }
                sum0 += average[m][n];
            }
        }
        int average0 = sum0 / (width * height);
        //System.out.print("图像的指纹为：");
        for (int n = 0; n < height; n++) {
            for (int m = 0; m < width; m++) {
                if (average[m][n] > average0)
                    average[m][n] = 1;
                else
                    average[m][n] = 0;
                rgbAverage[m + width * n] = average[m][n];
                //System.out.print(average[m][n]);
            }
        }
        //System.out.println();
        return rgbAverage;
    }

    /**
     * 计算相似度
     */
    public static double calcSimilarity(int[] rgbAverage1, int[] rgbAverage2, String... msg) {
        if (Objects.equals(Arrays.stream(rgbAverage1).sum(), 0) ||
                Objects.equals(Arrays.stream(rgbAverage2).sum(), 0)) {
            System.out.println("未加载图片");
            return 0;
        }
        double similarity = 0;
        for (int i = 0; i < 64; i++) {
            if (rgbAverage1[i] == rgbAverage2[i])
                similarity += 1.5625;
        }
        //System.out.printf("\n两张图像的相似度为：%s\n%n", similarity);
        log.info((msg.length > 0 ? msg[0] : "") + "两张图像的相似度为: " + similarity);
        return similarity;
    }

}
