package cn.fw.dalaran.service.biz;

import cn.fw.common.web.annotation.DisLock;
import cn.fw.dalaran.domain.db.config.EvlIndex;
import cn.fw.dalaran.domain.db.config.EvlSubIndex;
import cn.fw.dalaran.domain.enums.BizTypeEnum;
import cn.fw.dalaran.domain.enums.SubTypeEnum;
import cn.fw.dalaran.domain.param.EvlParams;
import cn.fw.dalaran.domain.vo.EvlIndexVO;
import cn.fw.dalaran.domain.vo.EvlSubIndexVO;
import cn.fw.dalaran.service.data.EvlIndexService;
import cn.fw.dalaran.service.data.EvlSubIndexService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author kurisu
 * @date 2021-11-18 11:30
 * @description 评价指标设置
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class EvlSettingService {
    private final EvlIndexService evlIndexService;
    private final EvlSubIndexService evlSubIndexService;
    private static final double BASE_RATE = 100D;

    /**
     * 查询评价指标列表
     *
     * @param configGroupId
     * @return
     */
    @DisLock(prefix = "':' + #this.getClass().getSimpleName()", key = "'queryEvl:' + #groupId", message = "操作过于频繁，请稍后重试")
    public List<EvlIndexVO> list(final Long configGroupId) {
        int count = evlIndexService.count(Wrappers.<EvlIndex>lambdaQuery()
                .eq(EvlIndex::getConfigGroupId, configGroupId)
        );
        if (!Objects.equals(count, BizTypeEnum.values().length)) {
            initEvlIndex(configGroupId);
        }
        List<EvlIndex> evlIndexList = evlIndexService.list(Wrappers.<EvlIndex>lambdaQuery()
                .eq(EvlIndex::getConfigGroupId, configGroupId)
        );
        List<EvlIndexVO> list = new ArrayList<>();
        for (EvlIndex evlIndex : evlIndexList) {
            EvlIndexVO vo = new EvlIndexVO();
            BeanUtils.copyProperties(evlIndex, vo);
            vo.setId(evlIndex.getId());
            list.add(vo);
        }
        return list;
    }

    /**
     * 查询评价子指标列表
     *
     * @param evlId
     * @return
     */
    @DisLock(prefix = "':' + #this.getClass().getSimpleName()", key = "'querySubEvl:' + #evlId", message = "操作过于频繁，请稍后重试")
    public List<EvlSubIndexVO> sublist(final Long evlId) {
        EvlIndex evlIndex = evlIndexService.getById(evlId);
        BV.notNull(evlIndex, () -> String.format("不存在id为: %s的父指标", evlId));
        int count = evlSubIndexService.count(Wrappers.<EvlSubIndex>lambdaQuery().eq(EvlSubIndex::getEvlId, evlId));
        if (!Objects.equals(count, SubTypeEnum.getByBizType(evlIndex.getType()).size())) {
            initEvlSubIndex(evlId, evlIndex.getType());
        }
        List<EvlSubIndex> evlSubIndexList = evlSubIndexService.list(Wrappers.<EvlSubIndex>lambdaQuery().eq(EvlSubIndex::getEvlId, evlId));
        List<EvlSubIndexVO> list = new ArrayList<>();
        for (EvlSubIndex evlSubIndex : evlSubIndexList) {
            EvlSubIndexVO vo = new EvlSubIndexVO();
            BeanUtils.copyProperties(evlSubIndex, vo);
            vo.setId(evlSubIndex.getId());
            list.add(vo);
        }
        return list;
    }

    /**
     * 修改指标占比
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean saveEvlIndex(List<EvlParams.EvlIndex> details) {
        BV.isTrue(Objects.equals(details.stream()
                .mapToDouble(item -> item.getRate().doubleValue())
                .sum(), BASE_RATE), "父指标占比之和不为100%, 请重新配置");
        final EvlIndex evlIndex1 = new EvlIndex();
        details.forEach(item -> {
            final EvlIndex evlIndex = evlIndexService.lambdaQuery()
                    .in(EvlIndex::getId, details.stream()
                            .map(EvlParams.EvlIndex::getId)
                            .collect(Collectors.toList())
                    ).list()
                    .stream()
                    .filter(item1 -> Objects.equals(item.getId(), item1.getId()))
                    .findFirst()
                    .orElse(new EvlIndex());
            if (Objects.nonNull(evlIndex.getId())) {
                evlIndex.setRate(item.getRate());
                evlIndexService.updateById(evlIndex);
            } else {
                evlIndex1.setId(item.getId());
                evlIndex1.setType(BizTypeEnum.ofValue(item.getType()));
                evlIndex1.setRate(item.getRate());
                evlIndexService.save(evlIndex1);
            }
        });
        return true;
    }

    /**
     * 修改子指标占比
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean saveEvlSubIndex(List<EvlParams.EvlIndex> details) {
        BV.isTrue(Objects.equals(details.stream()
                .mapToDouble(item -> item.getRate().doubleValue())
                .sum(), BASE_RATE), Objects.equals(details.get(0).getType(), 1) ? "[短视频]" : "[直播]" + "子指标占比之和不为100%, 请重新配置");
        final EvlSubIndex evlSubIndex1 = new EvlSubIndex();
        details.forEach(item -> {
            final EvlSubIndex evlSubIndex = evlSubIndexService.lambdaQuery()
                    .in(EvlSubIndex::getId, details.stream()
                            .map(EvlParams.EvlIndex::getId)
                            .collect(Collectors.toList())
                    ).list()
                    .stream()
                    .filter(item1 -> Objects.equals(item.getId(), item1.getId()))
                    .findFirst()
                    .orElse(new EvlSubIndex());
            if (Objects.nonNull(evlSubIndex.getId())) {
                evlSubIndex.setRate(item.getRate());
                evlSubIndexService.updateById(evlSubIndex);
            } else {
                evlSubIndex1.setId(item.getId());
                evlSubIndex1.setRate(item.getRate());
                evlSubIndexService.save(evlSubIndex1);
            }
        });
        return true;
    }

    /**
     * 初始化评价指标
     *
     * @param configGroupId
     */
    @Transactional(rollbackFor = Exception.class)
    protected void initEvlIndex(final Long configGroupId) {
        List<EvlIndex> oldList = evlIndexService.lambdaQuery()
                .eq(EvlIndex::getConfigGroupId, configGroupId)
                .list();
        evlIndexService.remove(Wrappers.<EvlIndex>lambdaQuery()
                .eq(EvlIndex::getConfigGroupId, configGroupId)
        );
        List<EvlIndex> evlIndexList = Arrays.stream(BizTypeEnum.values())
                .map(item -> {
                    EvlIndex evlIndex = oldList.stream()
                            .filter(item1 -> Objects.equals(item1.getType(), item))
                            .findFirst()
                            .orElse(new EvlIndex());
                    return EvlIndex.initBean(item, configGroupId, evlIndex.getRate());
                })
                .collect(Collectors.toList());
        evlIndexService.saveBatch(evlIndexList);
    }

    /**
     * 初始化评价子指标
     *
     * @param evlId
     */
    @Transactional(rollbackFor = Exception.class)
    protected void initEvlSubIndex(final Long evlId, BizTypeEnum bizTypeEnum) {
        List<EvlSubIndex> oldList = evlSubIndexService.lambdaQuery().eq(EvlSubIndex::getEvlId, evlId).list();
        evlSubIndexService.remove(Wrappers.<EvlSubIndex>lambdaQuery().eq(EvlSubIndex::getEvlId, evlId));
        List<EvlSubIndex> evlSubIndexList = SubTypeEnum.getByBizType(bizTypeEnum)
                .stream()
                .map(item -> {
                    EvlSubIndex evlSubIndex = oldList.stream()
                            .filter(item1 -> Objects.equals(item1.getSubType(), item))
                            .findFirst()
                            .orElse(new EvlSubIndex());
                    return EvlSubIndex.initBean(item, evlId, evlSubIndex.getRate());
                })
                .collect(Collectors.toList());
        evlSubIndexService.saveBatch(evlSubIndexList);
    }

}
